// largepi.cpp: Large integers used to compute pi.
//    To be linked with large.cpp.
// From: Ammeraal, L. (1997) STL for C++ Programmers,
//       Chichester: John Wiley.

#include <fstream>
#include <time.h>
#include <stdlib.h>
#include "large.h"

// For the computation of pi:
large Atan(uint k, const large &N)
{  // Computes N * atan(1.0/k) as a large integer
   large a = 0,
      w = N * k,
      zero = 0,
      k2 = k * k,
      i = -1,
      two = 2;
   cout << "k = " << k << endl;
   while (w != zero)
   {  a += (w /= k2)/(i += two);
      a -= (w /= k2)/(i += two);
   }
   return a;
}

void PiOutput(ostream &os, ofstream &ofile, 
   const large &x, int n)
{  vector<char> s;
   x.num2char(s);
   // Characters of x in reverse order in s.
   vector<char>::reverse_iterator i;
   int k = 0; 
   char ch;
   for (i=s.rbegin(); i != s.rend(); ++i)
   {  ch = *i;
      os << ch;
      if (ofile) ofile << ch;
      // k digits have been displayed after period.
      if (k == 0)
      {  os << "." << endl;
         if (ofile) ofile << "." << endl;
      }  else
      if (k % 50 == 0)
      {  os << endl;
         if (ofile) ofile << endl;
      }  else
      if (k % 10 == 0)
      {  os << " ";
         if (ofile) ofile << " ";
      }
      if (k++ == n) break;
   }
}

int main()
{  int n, m;
   cout << "Computation of pi. Number of decimals: ";
   cin >> n; m = n + 3;
   cout << 
      "Copy of output to file pi.txt desired (y/n)? ";
   char answer;
   cin >> answer;
   ofstream ofile;
   if (answer == 'Y' || answer == 'y') 
      ofile.open("pi.txt");
   large TenPower, Pi;
   clock_t tStart, tEnd;
   tStart = clock();
   TenPower = power(5, m); TenPower <<= m;
      // Faster than TenPower = power(10, m); 

   Pi = (Atan(18, TenPower * 48) 
       + Atan(57, TenPower * 32)
       - Atan(239, TenPower * 20))/1000;

   tEnd = clock();
   cout << "Digits of pi:" << endl;
   PiOutput(cout, ofile, Pi, n);
   cout << "\nTime: " << (tEnd - tStart) << " ticks\n";
   return 0;
}
